# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'plugins/replaygain2/ui_options_replaygain2.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ReplayGain2OptionsPage(object):
    def setupUi(self, ReplayGain2OptionsPage):
        ReplayGain2OptionsPage.setObjectName("ReplayGain2OptionsPage")
        ReplayGain2OptionsPage.resize(477, 577)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ReplayGain2OptionsPage.sizePolicy().hasHeightForWidth())
        ReplayGain2OptionsPage.setSizePolicy(sizePolicy)
        self.vboxlayout = QtWidgets.QVBoxLayout(ReplayGain2OptionsPage)
        self.vboxlayout.setContentsMargins(9, 9, 9, 9)
        self.vboxlayout.setSpacing(6)
        self.vboxlayout.setObjectName("vboxlayout")
        self.replay_gain = QtWidgets.QGroupBox(ReplayGain2OptionsPage)
        self.replay_gain.setObjectName("replay_gain")
        self.vboxlayout1 = QtWidgets.QVBoxLayout(self.replay_gain)
        self.vboxlayout1.setContentsMargins(9, 9, 9, 9)
        self.vboxlayout1.setSpacing(2)
        self.vboxlayout1.setObjectName("vboxlayout1")
        self.label = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.vboxlayout1.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.rsgain_command = QtWidgets.QLineEdit(self.replay_gain)
        self.rsgain_command.setObjectName("rsgain_command")
        self.horizontalLayout.addWidget(self.rsgain_command)
        self.rsgain_command_browse = QtWidgets.QToolButton(self.replay_gain)
        self.rsgain_command_browse.setObjectName("rsgain_command_browse")
        self.horizontalLayout.addWidget(self.rsgain_command_browse)
        self.vboxlayout1.addLayout(self.horizontalLayout)
        self.label_6 = QtWidgets.QLabel(self.replay_gain)
        self.label_6.setOpenExternalLinks(True)
        self.label_6.setTextInteractionFlags(QtCore.Qt.TextBrowserInteraction)
        self.label_6.setObjectName("label_6")
        self.vboxlayout1.addWidget(self.label_6)
        self.album_tags = QtWidgets.QCheckBox(self.replay_gain)
        self.album_tags.setChecked(True)
        self.album_tags.setObjectName("album_tags")
        self.vboxlayout1.addWidget(self.album_tags)
        self.true_peak = QtWidgets.QCheckBox(self.replay_gain)
        self.true_peak.setObjectName("true_peak")
        self.vboxlayout1.addWidget(self.true_peak)
        self.reference_loudness = QtWidgets.QCheckBox(self.replay_gain)
        self.reference_loudness.setObjectName("reference_loudness")
        self.vboxlayout1.addWidget(self.reference_loudness)
        self.label_5 = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.vboxlayout1.addWidget(self.label_5)
        self.target_loudness = QtWidgets.QSpinBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.target_loudness.sizePolicy().hasHeightForWidth())
        self.target_loudness.setSizePolicy(sizePolicy)
        self.target_loudness.setMinimum(-30)
        self.target_loudness.setMaximum(-5)
        self.target_loudness.setProperty("value", -18)
        self.target_loudness.setObjectName("target_loudness")
        self.vboxlayout1.addWidget(self.target_loudness)
        self.label_2 = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.vboxlayout1.addWidget(self.label_2)
        self.clip_mode = QtWidgets.QComboBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.clip_mode.sizePolicy().hasHeightForWidth())
        self.clip_mode.setSizePolicy(sizePolicy)
        self.clip_mode.setMaxCount(3)
        self.clip_mode.setObjectName("clip_mode")
        self.vboxlayout1.addWidget(self.clip_mode)
        self.label_3 = QtWidgets.QLabel(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.vboxlayout1.addWidget(self.label_3)
        self.max_peak = QtWidgets.QSpinBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_peak.sizePolicy().hasHeightForWidth())
        self.max_peak.setSizePolicy(sizePolicy)
        self.max_peak.setMinimum(-5)
        self.max_peak.setMaximum(0)
        self.max_peak.setObjectName("max_peak")
        self.vboxlayout1.addWidget(self.max_peak)
        self.label_4 = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.vboxlayout1.addWidget(self.label_4)
        self.opus_mode = QtWidgets.QComboBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.opus_mode.sizePolicy().hasHeightForWidth())
        self.opus_mode.setSizePolicy(sizePolicy)
        self.opus_mode.setMaxCount(3)
        self.opus_mode.setObjectName("opus_mode")
        self.vboxlayout1.addWidget(self.opus_mode)
        self.opus_m23 = QtWidgets.QCheckBox(self.replay_gain)
        self.opus_m23.setObjectName("opus_m23")
        self.vboxlayout1.addWidget(self.opus_m23)
        self.vboxlayout.addWidget(self.replay_gain)
        spacerItem = QtWidgets.QSpacerItem(263, 21, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.vboxlayout.addItem(spacerItem)

        self.retranslateUi(ReplayGain2OptionsPage)
        self.opus_mode.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(ReplayGain2OptionsPage)

    def retranslateUi(self, ReplayGain2OptionsPage):
        _translate = QtCore.QCoreApplication.translate
        self.replay_gain.setTitle(_translate("ReplayGain2OptionsPage", "ReplayGain 2.0 Settings"))
        self.label.setText(_translate("ReplayGain2OptionsPage", "Path to rsgain:"))
        self.rsgain_command.setText(_translate("ReplayGain2OptionsPage", "rsgain"))
        self.rsgain_command_browse.setText(_translate("ReplayGain2OptionsPage", "Browse..."))
        self.label_6.setText(_translate("ReplayGain2OptionsPage", "<a href=\"https://github.com/complexlogic/rsgain\">Download rsgain</a>"))
        self.album_tags.setText(_translate("ReplayGain2OptionsPage", "Calculate album gain/peak"))
        self.true_peak.setWhatsThis(_translate("ReplayGain2OptionsPage", "Use more accurate true peak calculations (trade for performance)"))
        self.true_peak.setText(_translate("ReplayGain2OptionsPage", "Use true peak"))
        self.reference_loudness.setText(_translate("ReplayGain2OptionsPage", "Write reference loudness tags"))
        self.label_5.setText(_translate("ReplayGain2OptionsPage", "Target Loudness (LUFS)"))
        self.label_2.setText(_translate("ReplayGain2OptionsPage", "Clipping Protection"))
        self.label_3.setText(_translate("ReplayGain2OptionsPage", "Max Peak (dB)"))
        self.label_4.setText(_translate("ReplayGain2OptionsPage", "Opus Files"))
        self.opus_m23.setText(_translate("ReplayGain2OptionsPage", "Always reference Opus R128_*_GAIN tags to -23 LUFS"))
