# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'plugins/format_performer_tags/ui_options_format_performer_tags.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FormatPerformerTagsOptionsPage(object):
    def setupUi(self, FormatPerformerTagsOptionsPage):
        FormatPerformerTagsOptionsPage.setObjectName("FormatPerformerTagsOptionsPage")
        FormatPerformerTagsOptionsPage.resize(561, 802)
        FormatPerformerTagsOptionsPage.setMinimumSize(QtCore.QSize(360, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(FormatPerformerTagsOptionsPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea = QtWidgets.QScrollArea(FormatPerformerTagsOptionsPage)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, -13, 529, 848))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gb_description = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.gb_description.setObjectName("gb_description")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.gb_description)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.format_description = QtWidgets.QLabel(self.gb_description)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.format_description.sizePolicy().hasHeightForWidth())
        self.format_description.setSizePolicy(sizePolicy)
        self.format_description.setWordWrap(True)
        self.format_description.setObjectName("format_description")
        self.verticalLayout_3.addWidget(self.format_description)
        self.verticalLayout_2.addWidget(self.gb_description)
        self.gb_word_groups = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.gb_word_groups.setObjectName("gb_word_groups")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.gb_word_groups)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.group_additonal = QtWidgets.QGroupBox(self.gb_word_groups)
        self.group_additonal.setObjectName("group_additonal")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.group_additonal)
        self.horizontalLayout.setContentsMargins(1, 1, 1, 1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.additional_rb_1 = QtWidgets.QRadioButton(self.group_additonal)
        self.additional_rb_1.setMaximumSize(QtCore.QSize(40, 16777215))
        self.additional_rb_1.setObjectName("additional_rb_1")
        self.horizontalLayout.addWidget(self.additional_rb_1)
        self.additional_rb_2 = QtWidgets.QRadioButton(self.group_additonal)
        self.additional_rb_2.setMaximumSize(QtCore.QSize(40, 16777215))
        self.additional_rb_2.setObjectName("additional_rb_2")
        self.horizontalLayout.addWidget(self.additional_rb_2)
        self.additional_rb_3 = QtWidgets.QRadioButton(self.group_additonal)
        self.additional_rb_3.setMaximumSize(QtCore.QSize(40, 16777215))
        self.additional_rb_3.setObjectName("additional_rb_3")
        self.horizontalLayout.addWidget(self.additional_rb_3)
        self.additional_rb_4 = QtWidgets.QRadioButton(self.group_additonal)
        self.additional_rb_4.setMaximumSize(QtCore.QSize(40, 16777215))
        self.additional_rb_4.setObjectName("additional_rb_4")
        self.horizontalLayout.addWidget(self.additional_rb_4)
        self.verticalLayout_5.addWidget(self.group_additonal)
        self.group_guest = QtWidgets.QGroupBox(self.gb_word_groups)
        self.group_guest.setObjectName("group_guest")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.group_guest)
        self.horizontalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.guest_rb_1 = QtWidgets.QRadioButton(self.group_guest)
        self.guest_rb_1.setMaximumSize(QtCore.QSize(40, 16777215))
        self.guest_rb_1.setObjectName("guest_rb_1")
        self.horizontalLayout_2.addWidget(self.guest_rb_1)
        self.guest_rb_2 = QtWidgets.QRadioButton(self.group_guest)
        self.guest_rb_2.setMaximumSize(QtCore.QSize(40, 16777215))
        self.guest_rb_2.setObjectName("guest_rb_2")
        self.horizontalLayout_2.addWidget(self.guest_rb_2)
        self.guest_rb_3 = QtWidgets.QRadioButton(self.group_guest)
        self.guest_rb_3.setMaximumSize(QtCore.QSize(40, 16777215))
        self.guest_rb_3.setObjectName("guest_rb_3")
        self.horizontalLayout_2.addWidget(self.guest_rb_3)
        self.guest_rb_4 = QtWidgets.QRadioButton(self.group_guest)
        self.guest_rb_4.setMaximumSize(QtCore.QSize(40, 16777215))
        self.guest_rb_4.setObjectName("guest_rb_4")
        self.horizontalLayout_2.addWidget(self.guest_rb_4)
        self.verticalLayout_5.addWidget(self.group_guest)
        self.group_solo = QtWidgets.QGroupBox(self.gb_word_groups)
        self.group_solo.setObjectName("group_solo")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.group_solo)
        self.horizontalLayout_3.setContentsMargins(1, 1, 1, 1)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.solo_rb_1 = QtWidgets.QRadioButton(self.group_solo)
        self.solo_rb_1.setMaximumSize(QtCore.QSize(40, 16777215))
        self.solo_rb_1.setObjectName("solo_rb_1")
        self.horizontalLayout_3.addWidget(self.solo_rb_1)
        self.solo_rb_2 = QtWidgets.QRadioButton(self.group_solo)
        self.solo_rb_2.setMaximumSize(QtCore.QSize(40, 16777215))
        self.solo_rb_2.setObjectName("solo_rb_2")
        self.horizontalLayout_3.addWidget(self.solo_rb_2)
        self.solo_rb_3 = QtWidgets.QRadioButton(self.group_solo)
        self.solo_rb_3.setMaximumSize(QtCore.QSize(40, 16777215))
        self.solo_rb_3.setObjectName("solo_rb_3")
        self.horizontalLayout_3.addWidget(self.solo_rb_3)
        self.solo_rb_4 = QtWidgets.QRadioButton(self.group_solo)
        self.solo_rb_4.setMaximumSize(QtCore.QSize(40, 16777215))
        self.solo_rb_4.setObjectName("solo_rb_4")
        self.horizontalLayout_3.addWidget(self.solo_rb_4)
        self.verticalLayout_5.addWidget(self.group_solo)
        self.group_vocals = QtWidgets.QGroupBox(self.gb_word_groups)
        self.group_vocals.setObjectName("group_vocals")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.group_vocals)
        self.horizontalLayout_4.setContentsMargins(1, 1, 1, 1)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.vocals_rb_1 = QtWidgets.QRadioButton(self.group_vocals)
        self.vocals_rb_1.setMinimumSize(QtCore.QSize(40, 20))
        self.vocals_rb_1.setMaximumSize(QtCore.QSize(40, 16777215))
        self.vocals_rb_1.setObjectName("vocals_rb_1")
        self.horizontalLayout_4.addWidget(self.vocals_rb_1)
        self.vocals_rb_2 = QtWidgets.QRadioButton(self.group_vocals)
        self.vocals_rb_2.setMaximumSize(QtCore.QSize(40, 16777215))
        self.vocals_rb_2.setObjectName("vocals_rb_2")
        self.horizontalLayout_4.addWidget(self.vocals_rb_2)
        self.vocals_rb_3 = QtWidgets.QRadioButton(self.group_vocals)
        self.vocals_rb_3.setMaximumSize(QtCore.QSize(40, 16777215))
        self.vocals_rb_3.setObjectName("vocals_rb_3")
        self.horizontalLayout_4.addWidget(self.vocals_rb_3)
        self.vocals_rb_4 = QtWidgets.QRadioButton(self.group_vocals)
        self.vocals_rb_4.setMaximumSize(QtCore.QSize(40, 16777215))
        self.vocals_rb_4.setObjectName("vocals_rb_4")
        self.horizontalLayout_4.addWidget(self.vocals_rb_4)
        self.verticalLayout_5.addWidget(self.group_vocals)
        self.verticalLayout_2.addWidget(self.gb_word_groups)
        self.gb_group_settings = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.gb_group_settings.setObjectName("gb_group_settings")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.gb_group_settings)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setVerticalSpacing(1)
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(self.gb_group_settings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1, QtCore.Qt.AlignLeft)
        self.label_3 = QtWidgets.QLabel(self.gb_group_settings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1, QtCore.Qt.AlignLeft)
        self.format_group_1_start_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_1_start_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_1_start_char.setObjectName("format_group_1_start_char")
        self.gridLayout.addWidget(self.format_group_1_start_char, 1, 1, 1, 1, QtCore.Qt.AlignHCenter)
        self.label = QtWidgets.QLabel(self.gb_group_settings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1, QtCore.Qt.AlignLeft)
        self.label_4 = QtWidgets.QLabel(self.gb_group_settings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 4, 0, 1, 1, QtCore.Qt.AlignLeft)
        self.format_group_1_sep_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_1_sep_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_1_sep_char.setObjectName("format_group_1_sep_char")
        self.gridLayout.addWidget(self.format_group_1_sep_char, 1, 2, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_1_end_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_1_end_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_1_end_char.setObjectName("format_group_1_end_char")
        self.gridLayout.addWidget(self.format_group_1_end_char, 1, 3, 1, 1, QtCore.Qt.AlignHCenter)
        self.label_5 = QtWidgets.QLabel(self.gb_group_settings)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 0, 1, 1, 1, QtCore.Qt.AlignHCenter)
        self.label_6 = QtWidgets.QLabel(self.gb_group_settings)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 0, 2, 1, 1, QtCore.Qt.AlignHCenter)
        self.label_7 = QtWidgets.QLabel(self.gb_group_settings)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 0, 3, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_2_start_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_2_start_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_2_start_char.setObjectName("format_group_2_start_char")
        self.gridLayout.addWidget(self.format_group_2_start_char, 2, 1, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_3_start_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_3_start_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_3_start_char.setObjectName("format_group_3_start_char")
        self.gridLayout.addWidget(self.format_group_3_start_char, 3, 1, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_4_start_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_4_start_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_4_start_char.setObjectName("format_group_4_start_char")
        self.gridLayout.addWidget(self.format_group_4_start_char, 4, 1, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_2_sep_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_2_sep_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_2_sep_char.setObjectName("format_group_2_sep_char")
        self.gridLayout.addWidget(self.format_group_2_sep_char, 2, 2, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_3_sep_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_3_sep_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_3_sep_char.setObjectName("format_group_3_sep_char")
        self.gridLayout.addWidget(self.format_group_3_sep_char, 3, 2, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_4_sep_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_4_sep_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_4_sep_char.setObjectName("format_group_4_sep_char")
        self.gridLayout.addWidget(self.format_group_4_sep_char, 4, 2, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_2_end_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_2_end_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_2_end_char.setObjectName("format_group_2_end_char")
        self.gridLayout.addWidget(self.format_group_2_end_char, 2, 3, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_3_end_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_3_end_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_3_end_char.setObjectName("format_group_3_end_char")
        self.gridLayout.addWidget(self.format_group_3_end_char, 3, 3, 1, 1, QtCore.Qt.AlignHCenter)
        self.format_group_4_end_char = QtWidgets.QLineEdit(self.gb_group_settings)
        self.format_group_4_end_char.setMaximumSize(QtCore.QSize(50, 16777215))
        self.format_group_4_end_char.setObjectName("format_group_4_end_char")
        self.gridLayout.addWidget(self.format_group_4_end_char, 4, 3, 1, 1, QtCore.Qt.AlignHCenter)
        self.verticalLayout_6.addLayout(self.gridLayout)
        self.verticalLayout_2.addWidget(self.gb_group_settings)
        self.gb_examples = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.gb_examples.setObjectName("gb_examples")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.gb_examples)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.example_instruments = QtWidgets.QLabel(self.gb_examples)
        self.example_instruments.setText("")
        self.example_instruments.setObjectName("example_instruments")
        self.verticalLayout_4.addWidget(self.example_instruments)
        self.example_vocals = QtWidgets.QLabel(self.gb_examples)
        self.example_vocals.setText("")
        self.example_vocals.setObjectName("example_vocals")
        self.verticalLayout_4.addWidget(self.example_vocals)
        self.verticalLayout_2.addWidget(self.gb_examples)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)

        self.retranslateUi(FormatPerformerTagsOptionsPage)
        QtCore.QMetaObject.connectSlotsByName(FormatPerformerTagsOptionsPage)

    def retranslateUi(self, FormatPerformerTagsOptionsPage):
        _translate = QtCore.QCoreApplication.translate
        FormatPerformerTagsOptionsPage.setWindowTitle(_translate("FormatPerformerTagsOptionsPage", "Form"))
        self.gb_description.setTitle(_translate("FormatPerformerTagsOptionsPage", "Format Performer Tags"))
        self.format_description.setText(_translate("FormatPerformerTagsOptionsPage", "<html><head/><body><p>These settings will determine the format for any <span style=\" font-weight:600;\">Performer</span> tags prepared. The format is divided into six parts: the performer; the instrument or &quot;vocals&quot;; and four user selectable sections for the extra information. This is set out as:</p><p align=\"center\"><span style=\" font-weight:600;\">[Section 1]</span>Instrument/Vocals<span style=\" font-weight:600;\">[Section 2][Section 3]</span>: Performer<span style=\" font-weight:600;\">[Section 4]</span></p><p>You can select the section in which each of the extra information words appears.</p><p>For each of the sections you can select the starting character(s), the character(s) separating entries, and the ending character(s). Note that leading or trailing spaces must be included in the settings and will not be automatically added. If no separator characters are entered, the items within a section will be automatically separated by a single space.</p><p>Please visit the repository on GitHub for <a href=\"https://github.com/rdswift/picard-plugins/blob/2.0_RDS_Plugins/plugins/format_performer_tags/docs/README.md\"><span style=\" text-decoration: underline; color:#0000ff;\">additional information</span></a>.</p></body></html>"))
        self.gb_word_groups.setTitle(_translate("FormatPerformerTagsOptionsPage", "Keyword Sections Assignment"))
        self.group_additonal.setTitle(_translate("FormatPerformerTagsOptionsPage", "Keyword: additional"))
        self.additional_rb_1.setText(_translate("FormatPerformerTagsOptionsPage", "1"))
        self.additional_rb_2.setText(_translate("FormatPerformerTagsOptionsPage", "2"))
        self.additional_rb_3.setText(_translate("FormatPerformerTagsOptionsPage", "3"))
        self.additional_rb_4.setText(_translate("FormatPerformerTagsOptionsPage", "4"))
        self.group_guest.setTitle(_translate("FormatPerformerTagsOptionsPage", "Keyword: guest"))
        self.guest_rb_1.setText(_translate("FormatPerformerTagsOptionsPage", "1"))
        self.guest_rb_2.setText(_translate("FormatPerformerTagsOptionsPage", "2"))
        self.guest_rb_3.setText(_translate("FormatPerformerTagsOptionsPage", "3"))
        self.guest_rb_4.setText(_translate("FormatPerformerTagsOptionsPage", "4"))
        self.group_solo.setTitle(_translate("FormatPerformerTagsOptionsPage", "Keyword: solo"))
        self.solo_rb_1.setText(_translate("FormatPerformerTagsOptionsPage", "1"))
        self.solo_rb_2.setText(_translate("FormatPerformerTagsOptionsPage", "2"))
        self.solo_rb_3.setText(_translate("FormatPerformerTagsOptionsPage", "3"))
        self.solo_rb_4.setText(_translate("FormatPerformerTagsOptionsPage", "4"))
        self.group_vocals.setTitle(_translate("FormatPerformerTagsOptionsPage", "All vocal type keywords"))
        self.vocals_rb_1.setText(_translate("FormatPerformerTagsOptionsPage", "1"))
        self.vocals_rb_2.setText(_translate("FormatPerformerTagsOptionsPage", "2"))
        self.vocals_rb_3.setText(_translate("FormatPerformerTagsOptionsPage", "3"))
        self.vocals_rb_4.setText(_translate("FormatPerformerTagsOptionsPage", "4"))
        self.gb_group_settings.setTitle(_translate("FormatPerformerTagsOptionsPage", "Section Display Settings"))
        self.label_2.setText(_translate("FormatPerformerTagsOptionsPage", "Section 2"))
        self.label_3.setText(_translate("FormatPerformerTagsOptionsPage", "Section 3"))
        self.format_group_1_start_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.label.setText(_translate("FormatPerformerTagsOptionsPage", "Section 1"))
        self.label_4.setText(_translate("FormatPerformerTagsOptionsPage", "Section 4"))
        self.format_group_1_sep_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_1_end_char.setText(_translate("FormatPerformerTagsOptionsPage", " "))
        self.format_group_1_end_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.label_5.setText(_translate("FormatPerformerTagsOptionsPage", "Start Char(s)"))
        self.label_6.setText(_translate("FormatPerformerTagsOptionsPage", "Sep Char(s)"))
        self.label_7.setText(_translate("FormatPerformerTagsOptionsPage", "End Char(s)"))
        self.format_group_2_start_char.setText(_translate("FormatPerformerTagsOptionsPage", ", "))
        self.format_group_2_start_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_3_start_char.setText(_translate("FormatPerformerTagsOptionsPage", " ("))
        self.format_group_3_start_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_4_start_char.setText(_translate("FormatPerformerTagsOptionsPage", " ("))
        self.format_group_4_start_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_2_sep_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_3_sep_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_4_sep_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_2_end_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_3_end_char.setText(_translate("FormatPerformerTagsOptionsPage", ")"))
        self.format_group_3_end_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.format_group_4_end_char.setText(_translate("FormatPerformerTagsOptionsPage", ")"))
        self.format_group_4_end_char.setPlaceholderText(_translate("FormatPerformerTagsOptionsPage", "(blank)"))
        self.gb_examples.setTitle(_translate("FormatPerformerTagsOptionsPage", "Examples"))
