# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'plugins/deezerart/options.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(420, 320)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.sizeLabel = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizeLabel.sizePolicy().hasHeightForWidth())
        self.sizeLabel.setSizePolicy(sizePolicy)
        self.sizeLabel.setObjectName("sizeLabel")
        self.gridLayout.addWidget(self.sizeLabel, 0, 0, 1, 1)
        self.size = QtWidgets.QComboBox(Form)
        self.size.setObjectName("size")
        self.gridLayout.addWidget(self.size, 0, 2, 1, 1)
        self.min_similarity_label = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.min_similarity_label.sizePolicy().hasHeightForWidth())
        self.min_similarity_label.setSizePolicy(sizePolicy)
        self.min_similarity_label.setObjectName("min_similarity_label")
        self.gridLayout.addWidget(self.min_similarity_label, 1, 0, 1, 2)
        self.min_similarity = QtWidgets.QSpinBox(Form)
        self.min_similarity.setMaximum(100)
        self.min_similarity.setObjectName("min_similarity")
        self.gridLayout.addWidget(self.min_similarity, 1, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 1, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.sizeLabel.setText(_translate("Form", "Cover size:"))
        self.min_similarity_label.setText(_translate("Form", "Minimal similarity for matches:"))
        self.min_similarity.setSuffix(_translate("Form", " %"))
        self.min_similarity.setPrefix(_translate("Form", " "))
