# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'plugins/bpm/ui_options_bpm.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_BPMOptionsPage(object):
    def setupUi(self, BPMOptionsPage):
        BPMOptionsPage.setObjectName("BPMOptionsPage")
        BPMOptionsPage.resize(611, 273)
        self.verticalLayout = QtWidgets.QVBoxLayout(BPMOptionsPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.bpm_options = QtWidgets.QGroupBox(BPMOptionsPage)
        self.bpm_options.setObjectName("bpm_options")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.bpm_options)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalWidget = QtWidgets.QWidget(self.bpm_options)
        self.verticalWidget.setMaximumSize(QtCore.QSize(500, 16777215))
        self.verticalWidget.setObjectName("verticalWidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.verticalWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.slider_parameter = QtWidgets.QSlider(self.verticalWidget)
        self.slider_parameter.setMinimum(1)
        self.slider_parameter.setMaximum(3)
        self.slider_parameter.setPageStep(1)
        self.slider_parameter.setOrientation(QtCore.Qt.Horizontal)
        self.slider_parameter.setTickPosition(QtWidgets.QSlider.TicksBelow)
        self.slider_parameter.setTickInterval(1)
        self.slider_parameter.setObjectName("slider_parameter")
        self.verticalLayout_3.addWidget(self.slider_parameter)
        self.slider_labels = QtWidgets.QHBoxLayout()
        self.slider_labels.setSpacing(6)
        self.slider_labels.setObjectName("slider_labels")
        self.slider_super_fast = QtWidgets.QLabel(self.verticalWidget)
        self.slider_super_fast.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.slider_super_fast.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.slider_super_fast.setObjectName("slider_super_fast")
        self.slider_labels.addWidget(self.slider_super_fast)
        self.slider_default = QtWidgets.QLabel(self.verticalWidget)
        self.slider_default.setToolTip("")
        self.slider_default.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.slider_default.setObjectName("slider_default")
        self.slider_labels.addWidget(self.slider_default)
        self.verticalLayout_3.addLayout(self.slider_labels)
        self.line = QtWidgets.QFrame(self.verticalWidget)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_3.addWidget(self.line)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.samplerate_label = QtWidgets.QLabel(self.verticalWidget)
        self.samplerate_label.setObjectName("samplerate_label")
        self.gridLayout.addWidget(self.samplerate_label, 2, 0, 1, 1)
        self.samplerate_value = QtWidgets.QLabel(self.verticalWidget)
        self.samplerate_value.setText("")
        self.samplerate_value.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.samplerate_value.setObjectName("samplerate_value")
        self.gridLayout.addWidget(self.samplerate_value, 2, 1, 1, 1)
        self.hop_s_value = QtWidgets.QLabel(self.verticalWidget)
        self.hop_s_value.setText("")
        self.hop_s_value.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.hop_s_value.setObjectName("hop_s_value")
        self.gridLayout.addWidget(self.hop_s_value, 1, 1, 1, 1)
        self.hop_s_label = QtWidgets.QLabel(self.verticalWidget)
        self.hop_s_label.setObjectName("hop_s_label")
        self.gridLayout.addWidget(self.hop_s_label, 1, 0, 1, 1)
        self.win_s_value = QtWidgets.QLabel(self.verticalWidget)
        self.win_s_value.setText("")
        self.win_s_value.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.win_s_value.setObjectName("win_s_value")
        self.gridLayout.addWidget(self.win_s_value, 0, 1, 1, 1)
        self.win_s_label = QtWidgets.QLabel(self.verticalWidget)
        self.win_s_label.setObjectName("win_s_label")
        self.gridLayout.addWidget(self.win_s_label, 0, 0, 1, 1)
        self.gridLayout.setColumnStretch(0, 4)
        self.verticalLayout_3.addLayout(self.gridLayout)
        self.verticalLayout_2.addWidget(self.verticalWidget)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.verticalLayout.addWidget(self.bpm_options)

        self.retranslateUi(BPMOptionsPage)
        QtCore.QMetaObject.connectSlotsByName(BPMOptionsPage)

    def retranslateUi(self, BPMOptionsPage):
        _translate = QtCore.QCoreApplication.translate
        self.bpm_options.setTitle(_translate("BPMOptionsPage", "BPM Analyze Parameters:"))
        self.slider_super_fast.setText(_translate("BPMOptionsPage", "Super Fast"))
        self.slider_default.setText(_translate("BPMOptionsPage", "Default"))
        self.samplerate_label.setText(_translate("BPMOptionsPage", "Samplerate:"))
        self.hop_s_label.setText(_translate("BPMOptionsPage", "Number of frames between two consecutive runs:"))
        self.win_s_label.setText(_translate("BPMOptionsPage", "Length of FFT:"))
