PLUGIN_NAME = "Add to Collection"
PLUGIN_AUTHOR = "Dvir Yitzchaki (dvirtz@gmail.com)"
PLUGIN_DESCRIPTION = "Adds any saved release to one of your user collections"
PLUGIN_VERSION = "0.1.1"
PLUGIN_API_VERSIONS = ["2.0"]
PLUGIN_LICENSE = ("MIT",)
PLUGIN_LICENSE_URL = "https://spdx.org/licenses/MIT.html"
PLUGIN_USER_GUIDE_URL = (
    "https://github.com/metabrainz/picard-plugins/blob/2.0/plugins/add_to_collection/README.md"
)
