# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'plugins/post_tagging_actions/actions_status.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_ActionsStatus(object):
    def setupUi(self, ActionsStatus):
        ActionsStatus.setObjectName("ActionsStatus")
        ActionsStatus.resize(94, 18)
        self.horizontalLayout = QtWidgets.QHBoxLayout(ActionsStatus)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.actions_count = QtWidgets.QLabel(ActionsStatus)
        self.actions_count.setMinimumSize(QtCore.QSize(35, 0))
        self.actions_count.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.actions_count.setObjectName("actions_count")
        self.horizontalLayout.addWidget(self.actions_count)
        self.label = QtWidgets.QLabel(ActionsStatus)
        self.label.setText("")
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)

        self.retranslateUi(ActionsStatus)
        QtCore.QMetaObject.connectSlotsByName(ActionsStatus)

    def retranslateUi(self, ActionsStatus):
        _translate = QtCore.QCoreApplication.translate
        ActionsStatus.setWindowTitle(_translate("ActionsStatus", "Form"))
        self.actions_count.setText(_translate("ActionsStatus", "0"))
        self.label.setToolTip(_translate("ActionsStatus", "Remaining Actions"))
