# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'plugins/acousticbrainz/ui_options_acousticbrainz_tags.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AcousticBrainzOptionsPage(object):
    def setupUi(self, AcousticBrainzOptionsPage):
        AcousticBrainzOptionsPage.setObjectName("AcousticBrainzOptionsPage")
        AcousticBrainzOptionsPage.setEnabled(True)
        AcousticBrainzOptionsPage.resize(527, 443)
        AcousticBrainzOptionsPage.setWindowTitle("")
        self.verticalLayout = QtWidgets.QVBoxLayout(AcousticBrainzOptionsPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.acousticbrainzTags_groupBox = QtWidgets.QGroupBox(AcousticBrainzOptionsPage)
        self.acousticbrainzTags_groupBox.setObjectName("acousticbrainzTags_groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.acousticbrainzTags_groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.add_simplemood = QtWidgets.QCheckBox(self.acousticbrainzTags_groupBox)
        self.add_simplemood.setObjectName("add_simplemood")
        self.verticalLayout_2.addWidget(self.add_simplemood)
        self.simplemood_tagname_label = QtWidgets.QLabel(self.acousticbrainzTags_groupBox)
        self.simplemood_tagname_label.setObjectName("simplemood_tagname_label")
        self.verticalLayout_2.addWidget(self.simplemood_tagname_label)
        self.simplemood_tagname = QtWidgets.QLineEdit(self.acousticbrainzTags_groupBox)
        self.simplemood_tagname.setObjectName("simplemood_tagname")
        self.verticalLayout_2.addWidget(self.simplemood_tagname)
        self.add_simplegenre = QtWidgets.QCheckBox(self.acousticbrainzTags_groupBox)
        self.add_simplegenre.setObjectName("add_simplegenre")
        self.verticalLayout_2.addWidget(self.add_simplegenre)
        self.simplegenre_tagname_label = QtWidgets.QLabel(self.acousticbrainzTags_groupBox)
        self.simplegenre_tagname_label.setObjectName("simplegenre_tagname_label")
        self.verticalLayout_2.addWidget(self.simplegenre_tagname_label)
        self.simplegenre_tagname = QtWidgets.QLineEdit(self.acousticbrainzTags_groupBox)
        self.simplegenre_tagname.setObjectName("simplegenre_tagname")
        self.verticalLayout_2.addWidget(self.simplegenre_tagname)
        self.add_keybpm = QtWidgets.QCheckBox(self.acousticbrainzTags_groupBox)
        self.add_keybpm.setObjectName("add_keybpm")
        self.verticalLayout_2.addWidget(self.add_keybpm)
        self.add_fullhighlevel = QtWidgets.QCheckBox(self.acousticbrainzTags_groupBox)
        self.add_fullhighlevel.setObjectName("add_fullhighlevel")
        self.verticalLayout_2.addWidget(self.add_fullhighlevel)
        self.add_sublowlevel = QtWidgets.QCheckBox(self.acousticbrainzTags_groupBox)
        self.add_sublowlevel.setObjectName("add_sublowlevel")
        self.verticalLayout_2.addWidget(self.add_sublowlevel)
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        self.verticalLayout_2.addItem(spacerItem)
        self.sublowlevel_descr = QtWidgets.QLabel(self.acousticbrainzTags_groupBox)
        self.sublowlevel_descr.setTextFormat(QtCore.Qt.RichText)
        self.sublowlevel_descr.setObjectName("sublowlevel_descr")
        self.verticalLayout_2.addWidget(self.sublowlevel_descr)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addWidget(self.acousticbrainzTags_groupBox)

        self.retranslateUi(AcousticBrainzOptionsPage)
        QtCore.QMetaObject.connectSlotsByName(AcousticBrainzOptionsPage)

    def retranslateUi(self, AcousticBrainzOptionsPage):
        _translate = QtCore.QCoreApplication.translate
        self.acousticbrainzTags_groupBox.setTitle(_translate("AcousticBrainzOptionsPage", "AcousticBrainz Tags"))
        self.add_simplemood.setText(_translate("AcousticBrainzOptionsPage", "Add simple Mood tags"))
        self.simplemood_tagname_label.setText(_translate("AcousticBrainzOptionsPage", "Mood tag name:"))
        self.add_simplegenre.setText(_translate("AcousticBrainzOptionsPage", "Add simple Genre tags"))
        self.simplegenre_tagname_label.setText(_translate("AcousticBrainzOptionsPage", "Genre tag name:"))
        self.add_keybpm.setText(_translate("AcousticBrainzOptionsPage", "Add simple BPM and Key tags"))
        self.add_fullhighlevel.setText(_translate("AcousticBrainzOptionsPage", "Add all highlevel AcousticBrainz tags"))
        self.add_sublowlevel.setText(_translate("AcousticBrainzOptionsPage", "Add a subset of the lowlevel AcousticBrainz tags"))
        self.sublowlevel_descr.setText(_translate("AcousticBrainzOptionsPage", "<html><head/><body><p>The low level subset include:</p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">rhythm:bpm</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">tonal:chords_change_rate</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">tonal:chords_key</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">tonal:chords_scale</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">tonal:key_key</li><li style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">tonal:key_scale</li></ul></body></html>"))
